/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/02/05 16:19:45
 */

import { DOMParser } from '@xmldom/xmldom'
import svgpath from 'svgpath'

import { PATH_DECIMAL } from '../config.js'
import { normalizePath, pathify } from './path.js'
import { generateAmendTrans } from './view_box.js'

/**
 * 翻转svgpath
 * @param  {string} path     path序列
 * @return {string}         反转后的path序列
 */
export function reversal(path) {
  return svgpath(path).scale(1, -1).round(PATH_DECIMAL).toString()
}

/**
 * 转换svg为一个path,并且按照目标高度，偏移进行拉伸
 * @param  {string} svg     原始svg
 * @param  {object} options.targetHeight 目标高度
 * @return {object}         path序列和viewbox
 */
export function normalizeSvg(svg, options) {
  let svgDocNode = new DOMParser().parseFromString(
    pathify(svg),
    'application/xml'
  )
  let svgNode = svgDocNode.getElementsByTagName('svg')[0]

  //解决所有的变换，生成一个path
  let path = normalizePath(svgNode)

  let trans = svgpath(path)

  //根据目标viewbox进行变换
  let targetHeight = options.targetHeight

  if (targetHeight) {
    let viewObj = generateAmendTrans(svgNode, targetHeight)
    viewObj.transforms.forEach(function (viewTrans) {
      trans[viewTrans[0]].apply(trans, viewTrans[1])
    })
  }

  return {
    viewbox: viewObj.targetViewbox,
    path: trans.round(PATH_DECIMAL).toString()
  }
}
