/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/02/05 16:04:06
 */

function getViewBox(svgNode) {
  if (!svgNode) {
    return
  }

  let width, height
  let viewbox = svgNode.getAttribute('viewBox').replace(',', ' ').split(' ')

  if (viewbox && viewbox.length === 4) {
    return viewbox.map(v => +v)
  }

  width = +svgNode.getAttribute('width')
  height = +svgNode.getAttribute('height')
  if (width != 0 && height != 0) {
    return [0, 0, width, height]
  }
}

function getViewPort(svgNode) {
  let width, height, viewbox
  width = +svgNode.getAttribute('width')
  height = +svgNode.getAttribute('height')

  viewbox = svgNode.getAttribute('viewBox').replace(',', ' ').split(' ')
  if (!width) {
    width = +viewbox[2]
  }
  if (!height) {
    height = +viewbox[3]
  }

  return [width, height]
}

// 根据 targetHeight 缩放 width
// 如: normalizeViewport([1024, 2048], 1024)
// #=> [0,0,512, 1024]
function getTargetViewbox(viewport, targetHeight) {
  let width = viewport[0]
  let height = viewport[1]

  if (height != targetHeight) {
    width = ~~((targetHeight / height) * width)
    height = targetHeight
  }

  return [0, 0, width, height]
}

function normalizeXY(viewbox, targetViewbox) {
  let x, y, targetX, targetY

  x = viewbox[0]
  y = viewbox[1]
  targetX = targetViewbox[0]
  targetY = targetViewbox[1]
  //可能会有x,y偏移的情况，所以这边需要做出相应的转换
  if (x != targetX || y != targetY) {
    return [['translate', [targetX - x, targetY - y]]]
  }
  return []
}

function normalizeWidthHeight(viewbox, targetViewbox) {
  let width, height, targetWidth, targetHeight, widthScale, heightScale

  width = viewbox[2]
  height = viewbox[3]
  targetWidth = targetViewbox[2]
  targetHeight = targetViewbox[3]

  widthScale = width / targetWidth
  heightScale = height / targetHeight
  //比较正常的等比缩放
  if (widthScale == heightScale) {
    return [['scale', [1 / widthScale, 1 / widthScale]]]
  }

  //下面是不等比缩放
  //不等比缩放后
  let maxScale = Math.max(widthScale, heightScale)
  let transforms = [],
    newWidth,
    newHeight

  if (widthScale < heightScale) {
    newWidth = targetWidth * maxScale
    transforms.push(['translate', [(newWidth - width) / 2, 0]])
  } else if (widthScale > heightScale) {
    newHeight = newHeight * maxScale
    transforms.push(['translate', [0, (newHeight - height) / 2]])
  }

  transforms.push(['scale', [1 / maxScale, 1 / maxScale]])

  return transforms
}

//返回需要进行的转换数组
function normalizeViewBox(viewbox, targetViewbox) {
  return normalizeXY(viewbox, targetViewbox).concat(
    normalizeWidthHeight(viewbox, targetViewbox)
  )
}

export function generateAmendTrans(svgNode, targetHeight) {
  let viewport = getViewPort(svgNode)
  let viewbox = getViewBox(svgNode)
  let targetViewbox = getTargetViewbox(viewport, targetHeight)

  return {
    transforms: normalizeViewBox(viewbox, targetViewbox),
    targetViewbox: targetViewbox
  }
}
